import { useEffect, useMemo, useState } from "react";
import styles from "./index.module.css";
import { DownOutlined } from "@ant-design/icons";
import BasicModal from "@/components/BasicModal";
import { useBoolean } from "ahooks";
import { Button, Empty, Input, Space, Tag, Tree, TreeDataNode } from "antd";
import setting from "@/assets/images/setting/setting.png";
import service from "@/service";
import { isEqual } from "lodash";

interface UserSelectProps {
  value: string[];
  onChange: (values: string[]) => void;
}

const UserSelect = (props: UserSelectProps) => {
  const { value, onChange } = props;
  const [selected, setSelected] = useState<string[]>([]);

  useEffect(() => {
    if (isEqual(value, selected)) {
      return;
    } else {
      setSelected(value);
    }
  }, [value]);

  /** 开启选人modal */
  const [visible, { toggle }] = useBoolean(false);
  const [dataSource, setDataSource] = useState<API.StaffInfo[]>([]);
  const [userTree, setUserTree] = useState<TreeDataNode[]>([]);

  const buildTree = (data: API.StaffInfo[]) => {
    const treeData = data?.map((item) => {
      return {
        title: item.realName,
        key: item.id,
        value: item.id,
      };
    });
    setUserTree(treeData);
    return treeData;
  };

  useEffect(() => {
    buildTree(dataSource);
  }, [dataSource]);

  useEffect(() => {
    service.commonService.getAllUser().then((res) => {
      if (res.code === 200) {
        setDataSource(res.data);
      }
    });
  }, []);

  const onCheck = (_: string[], e: any) => {
    if (e.checked) {
      setSelected([...selected, e.node.value]);
    } else setSelected(selected.filter((v) => v !== e.node.value));
  };

  const selectedInfo = useMemo(() => {
    return (
      selected?.map((item) => {
        return dataSource.find((data) => data.id === item);
      }) ?? []
    );
  }, [selected, dataSource]);

  const onSearch = (searchStr: string) => {
    if (searchStr.length === 0) {
      buildTree(dataSource);
    }

    const list = dataSource.filter((item) => {
      return item.realName.includes(searchStr);
    });
    buildTree(list);
  };

  const submit = () => {
    if (onChange && typeof onChange === "function") {
      onChange(selected);
      toggle();
    }
  };

  const tagClose = (item: API.StaffInfo) => {
    setSelected(selected.filter((v) => v !== item.id));
  };

  return (
    <>
      <div className={styles.selectBtn} onClick={toggle}>
        {selected && selected?.length > 0 ? (
          <div>{selectedInfo.map((v) => v?.realName).join(",")}</div>
        ) : (
          <div>请选择</div>
        )}
        <DownOutlined />
      </div>
      <BasicModal
        visible={visible}
        width={1100}
        height={700}
        zIndex={1100}
        onClose={toggle}
        contentStyle={{ borderRadius: 10 }}
      >
        <div className={styles.container}>
          <div className={styles.header}>
            <img src={setting} style={{ width: 46, height: 46 }} alt=""></img>
            <div className={styles.headerTitle}>选择人员</div>
          </div>
          <div className={styles.modalBody}>
            <div className={styles.leftCard}>
              <Input.Search
                style={{ marginBottom: 16 }}
                placeholder="请输入人员名称"
                onSearch={onSearch}
              ></Input.Search>
              <div className={styles.treeContent}>
                <Tree
                  checkable
                  treeData={userTree}
                  checkedKeys={selected}
                  onCheck={onCheck as any}
                />
              </div>
            </div>
            <div className={styles.rightCard}>
              <div>选中的人员</div>
              <div style={{ marginTop: 8 }}>
                {selectedInfo?.map((item) => {
                  return (
                    <Tag
                      closeIcon
                      style={{ marginBottom: 8 }}
                      onClose={() => tagClose(item as API.StaffInfo)}
                    >
                      {item?.realName}
                    </Tag>
                  );
                })}
                {selectedInfo?.length === 0 && (
                  <div>
                    <Empty description="暂未选中人员"></Empty>
                  </div>
                )}
              </div>
            </div>
          </div>
          <div className={styles.footer}>
            <Space>
              <Button type="primary" onClick={submit}>
                确定
              </Button>
            </Space>
          </div>
        </div>
      </BasicModal>
    </>
  );
};

export default UserSelect;
