import { Property, getCompInfo, getPropMap } from "../decorator";

import {
  ComponentsType,
  KeyOf,
  ObjectConfigMetadata,
  ObjectMetadata,
  PropInfo,
  PropInfoMap,
} from "../interfaces";
import { Rule } from "antd/es/form";
import { generateUUID } from "../utils";
/**
 * 获取属性默认值
 */
function defaults(info: PropInfo): unknown {
  if (typeof info.def != "function") return info.def;
  return info.def();
}

/** 基础控件 */
export default class BaseFormComponent<
  T extends BaseConfig = any,
  U extends ComponentsType = ComponentsType
> {
  /** 获取对象的静态元数据 */
  static get info(): ObjectMetadata {
    return {
      type: this,
      name: this.name,
      properties: getPropMap(this),
      component: getCompInfo(this),
    };
  }

  groupId = "";

  /** 表单项标签 */
  @Property({
    def: "文本",
    desc: "字段名称",
    type: "TextInput",
    config: {
      placeholder: "请输入表单文本",
    },
  })
  label!: string;

  /** 对应的组件类型 */
  componentType!: U;
  /** 表单项key，生成规则就使用 uuid 随机生成吧 */
  key!: string;

  constructor() {
    const objInfo = new.target.info as ObjectMetadata<this>;

    for (const k in objInfo.properties) {
      const key = k as unknown as KeyOf<this>;
      const info = objInfo.properties[key];
      this[key] = defaults(info) as this[typeof key];
    }
    this.componentType = this.constructor.name as U;
    /** 表单key值随机分配 */
    this.key = generateUUID();
  }

  /** 设置值 */
  setValues(v: Record<string, any>) {
    for (const k in v) {
      if (this[k as KeyOf<BaseFormComponent>] === undefined) {
        /** 不接受不存在的值 */
        continue;
      }

      this[k as KeyOf<BaseFormComponent>] = v[k];
    }
  }
  toJsonValidationRules() {
    const obj = this;
    //@ts-ignore
    if (obj.validationRules?.[0]?.pattern) {
      const rule = [
        {
          ...obj.validationRules?.[0],
          //@ts-ignore
          pattern: obj.validationRules[0].pattern.toString(),
        },
      ];
      return JSON.stringify(rule);
    }
    return JSON.stringify(obj.validationRules);
  }
  /** 序列化 */
  toJson() {
    return JSON.stringify(this);
  }

  /** 获取静态元数据 */
  meta(): ObjectMetadata<this> {
    const config = this.componentOption?.meta();
    const compInfo = (this.constructor as typeof BaseFormComponent)
      .info as ObjectMetadata<this>;
    compInfo.properties = {
      ...compInfo.properties,
      ...config?.properties,
    };

    return compInfo;
  }
  /** 获取全部元数据 */
  info(): ObjectMetadata<this>;

  /**
   * 获取元数据
   */
  info(k?: KeyOf<this>): unknown {
    const meta = this.meta();
    if (k) {
      if (meta.properties[k]) {
        return { ...meta.properties[k] };
      }
    }
    const propList = {} as PropInfoMap;
    for (const key in meta.properties) {
      const staticPropInfo = meta.properties[key];
      propList[key] = {
        ...staticPropInfo,
      };
    }
    return {
      propList,
    };
  }

  /** 组件的config 配置 */
  componentOption?: T;
  /** 表单的检验规则 */
  validationRules = [] as Rule[];
}

/** config 基类 */
export class BaseConfig {
  /** 获取对象的静态元数据 */
  static get info(): ObjectConfigMetadata {
    return {
      properties: getPropMap(this),
    };
  }
  init() {
    const objInfo = (this.constructor as any)
      .info as ObjectConfigMetadata<this>;
    for (const k in objInfo.properties) {
      const key = k;
      const info = objInfo.properties[key];
      this[key] = defaults(info) as this[typeof key];
    }
  }
  update(v: Record<string, any>) {
    for (const k in v) {
      //@ts-ignore
      this[k] = v[k];
    }
    return this;
  }

  /** 获取静态元数据 */
  meta() {
    return (this.constructor as typeof BaseConfig).info;
  }
}
