import BasicModal from "@/components/BasicModal";
import React, { useEffect, useState } from "react";
import styles from "./index.module.css";
import wodeshengqing from "@/assets/images/applyRecord/wodeshengqing.png";
import { Button, Flex, Space, message } from "antd";
import Card from "./components/card";

import LabelText from "./components/labelText";

import Content from "./content";
import Attachment from "./attachment";
import ProcessTimeLine from "./timeLine";
import Review from "./review";
import { VerticalAlignBottomOutlined } from "@ant-design/icons";
import service from "@/service";

import { composeImgUrl } from "@/config/util";

type EditMode = "review" | "check";

interface ApplicationMaterialProps {
  visible: boolean;
  mode?: EditMode;
  onClose: () => void;
  actionObj: any;
}

/**
 * 下载文件
 * @param {String} path - 下载地址/下载请求地址。
 * @param {String} name - 下载文件的名字/重命名（考虑到兼容性问题，最好加上后缀名）
 */
const downloadFile = (path: string, name: string) => {
  const xhr = new XMLHttpRequest();
  xhr.open("get", path);
  xhr.responseType = "blob";
  xhr.send();
  xhr.onload = function () {
    if (this.status === 200 || this.status === 304) {
      // 如果是IE10及以上，不支持download属性，采用msSaveOrOpenBlob方法，但是IE10以下也不支持msSaveOrOpenBlob
      if ("msSaveOrOpenBlob" in navigator) {
        // navigator.msSaveOrOpenBlob(this.response, name);
        //@ts-ignore
        navigator.msSaveOrOpenBlob(this.response, name);
        return;
      }
      // const blob = new Blob([this.response], { type: xhr.getResponseHeader('Content-Type') });
      // const url = URL.createObjectURL(blob);
      const url = URL.createObjectURL(this.response);
      const a = document.createElement("a");
      a.style.display = "none";
      a.href = url;
      a.download = name;
      document.body.appendChild(a);
      a.click();
      document.body.removeChild(a);
      URL.revokeObjectURL(url);
    }
  };
};

const Header = ({ mode }: { mode: EditMode }) => {
  return (
    <Flex align="center" gap={12} className={styles.header}>
      <img src={wodeshengqing} alt="" style={{ width: 46, height: 46 }}></img>
      <div className={styles.modalTitle}>
        {mode === "review" ? "办理" : "我的申请"}
      </div>
    </Flex>
  );
};

const ApplicationMaterial: React.FC<ApplicationMaterialProps> = (props) => {
  const { visible = false, onClose, mode = "check", actionObj } = props;
  const [configSource, setConfigSource] = useState<any>({});
  const [detail, setDetail] = useState<any>({});

  const init = () => {
    if (mode === "review") {
      service.managementService
        .getApplicationFormDetail(actionObj.id)
        .then((res) => {
          if (res.code === 200) {
            setDetail(res.data);
          }
        });
    } else {
      service.applyService
        .getApplicationFormDetail(actionObj.id)
        .then((res) => {
          if (res.code === 200) {
            setDetail(res.data);
          }
        });
    }

    service.settingService.getTemplateConfig(actionObj.thingId).then((res) => {
      if (res.code === 200) {
        setConfigSource(res.data);
      }
    });
  };

  useEffect(() => {
    if (visible) {
      init();
    }
  }, [actionObj]);
  const findName = (key: string) => {
    return (
      configSource?.matterContentList?.find((v: any) => {
        return v.fieldKey === key;
      })?.label ?? "未命名文件"
    );
  };
  const downloadAll = () => {
    let canDownload =
      detail?.reportContents?.filter((v: any) => {
        return v.downFlag && v.content?.length > 0;
      }) ?? [];

    if (canDownload.length === 0) {
      message.warning("无可下载内容");
      return;
    }
    // console.log(canDownload);

    canDownload.forEach((v: any) => {
      downloadFile(composeImgUrl(v.content), findName(v.contentId));
    });
  };
  return (
    <BasicModal
      visible={visible}
      width={1778}
      contentStyle={{ borderRadius: 10 }}
      onClose={onClose}
    >
      <div className={styles.container}>
        <Header mode={mode}></Header>
        <div className={styles.modalBody}>
          <Card
            title="申请单位"
            bg="linear-gradient(90deg, rgba(80, 116, 252, 0.7) 0%, rgba(19, 80, 255, 0.7) 47.83%, rgba(0, 36, 255, 0.7) 100%)"
          >
            <div className={styles.infoWrapper}>
              <Space align="center" size="large" style={{ marginBottom: 30 }}>
                <div className={styles.infoTitle}>{detail?.subjectName}</div>
                {/* <Space>
                  <div className={styles.tag}>李政声</div>
                  <div className={styles.tag}>干洗店</div>
                </Space> */}
              </Space>
              <Flex wrap="wrap" gap={16}>
                <LabelText
                  label="组织机构代码"
                  text={detail?.subjectCode}
                ></LabelText>
                <LabelText
                  label="联系人"
                  text={detail?.subjectContact}
                ></LabelText>
                <LabelText
                  label="单位登记地址"
                  text={detail?.subjectAddress}
                ></LabelText>
                <LabelText
                  label="联系电话"
                  text={detail?.subjectPhone}
                ></LabelText>
              </Flex>
            </div>
          </Card>

          <Card
            title={mode === "check" ? "办理进度" : "办理经过"}
            bg="#fff"
            contentStyle={
              mode === "review"
                ? {
                    padding: 0,
                  }
                : {}
            }
          >
            {mode === "review" ? (
              <Review
                actionObj={actionObj}
                detail={detail}
                configSource={configSource}
              />
            ) : (
              <ProcessTimeLine detail={actionObj} />
            )}
          </Card>

          <Card title="申请内容" bg="#fff">
            <Content configSource={configSource} data={detail} />
          </Card>

          <Card
            title="申请附件"
            bg="#fff"
            headerExtra={
              <Button
                type="text"
                style={{ color: "rgba(46, 108, 255, 1)" }}
                onClick={downloadAll}
              >
                一键下载附件
                <VerticalAlignBottomOutlined />
              </Button>
            }
          >
            <Attachment configSource={configSource} data={detail} />
          </Card>
        </div>
      </div>
    </BasicModal>
  );
};

export default ApplicationMaterial;
